@@ -3,7 +3,7 @@
 *                     A p p l i c a t i o n   O b j e c t                       *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2005 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXApp.h,v 1.168 2003/10/27 23:59:21 fox Exp $                            *
+* $Id: FXApp.h,v 1.195.2.2 2005/03/18 05:36:12 fox Exp $                            *
 ********************************************************************************/
 #ifndef FXAPP_H
 #define FXAPP_H
@@ -100,6 +100,7 @@ enum FXDefaultCursor {
   DEF_CORNERSE_CURSOR,                  /// South-east cursor
   DEF_CORNERSW_CURSOR,                  /// South-west cursor
   DEF_HELP_CURSOR,                      /// Help arrow cursor
+  DEF_HAND_CURSOR,                      /// Hand cursor
   DEF_ROTATE_CURSOR                     /// Rotate cursor
   };
 
@@ -155,6 +156,7 @@ class FXAPI FXApp : public FXObject {
   friend class FXGLContext;
   friend class FXDC;
   friend class FXDCWindow;
+  friend class FXDockHandler;
 
 private:
 
@@ -189,13 +191,14 @@ private:
   FXint            nsignals;            // Number of signals
   FXFont          *normalFont;          // Normal font
   FXFont          *stockFont;           // Stock font
-  FXuint           maxcolors;           // Maximum number of colors to allocate
+  FXMutex          appMutex;            // Application wide mutex
   FXEvent          event;               // Event
   FXInput         *inputs;              // Input file descriptors being watched
   FXint            ninputs;             // Number of inputs
   FXint            maxinput;            // Maximum input number
   FXuchar         *ddeData;             // DDE array
   FXuint           ddeSize;             // DDE array size
+  FXuint           maxcolors;           // Maximum number of colors to allocate
   FXuint           typingSpeed;         // Typing speed
   FXuint           clickSpeed;          // Double click speed
   FXuint           scrollSpeed;         // Scroll speed
@@ -217,8 +220,10 @@ private:
   FXColor          selbackColor;        // Select background color
   FXColor          tipforeColor;        // Tooltip foreground color
   FXColor          tipbackColor;        // Tooltip background color
+  FXColor          selMenuTextColor;    // Select foreground color in menus
+  FXColor          selMenuBackColor;    // Select background color in menus
   FXCursor        *waitCursor;          // Current wait cursor
-  FXuint           waitcount;           // Number of times wait cursor was called
+  FXuint           waitCount;           // Number of times wait cursor was called
   FXCursor        *cursor[DEF_ROTATE_CURSOR+1];
   FXint                appArgc;         // Argument count
   const FXchar *const *appArgv;         // Argument vector
@@ -240,6 +245,9 @@ private:
   FXID             wmNetState;          // Extended Window Manager window state
   FXID             wmNetHMaximized;     // Extended Window Manager horizontally maximized
   FXID             wmNetVMaximized;     // Extended Window Manager vertically maximized
+  FXID             embedAtom;           // XEMBED support
+  FXID             embedInfoAtom;       // XEMBED info support
+  FXID             timestampAtom;       // Server time
   FXID             ddeTargets;          // DDE targets atom
   FXID             ddeAtom;             // DDE exchange atom
   FXID             ddeDelete;           // DDE delete target atom
@@ -276,6 +284,7 @@ private:
   FXbool           xdndStatusReceived;  // XDND received at least one status
   FXbool           xdndWantUpdates;     // XDND target wants new positions while in rect
   FXRectangle      xdndRect;            // XDND rectangle bounding target
+  FXint            xrreventbase;        // XRR event base
   FXID             stipples[23];        // Standard stipple patterns
   void            *r_fds;               // Set of file descriptors for read
   void            *w_fds;               // Set of file descriptors for write
@@ -317,22 +326,22 @@ private:
   FXApp &operator=(const FXApp&);
   static void signalhandler(int sig);
   static void immediatesignalhandler(int sig);
-  void dragdropSetData(const FXWindow* window,FXDragType type,FXuchar* data,FXuint size);
-  void dragdropGetData(const FXWindow* window,FXDragType type,FXuchar*& data,FXuint& size);
-  void dragdropGetTypes(const FXWindow* window,FXDragType*& types,FXuint& numtypes);
+  void leaveWindow(FXWindow *window,FXWindow *ancestor);
+  void enterWindow(FXWindow *window,FXWindow *ancestor);
   void selectionSetData(const FXWindow* window,FXDragType type,FXuchar* data,FXuint size);
   void selectionGetData(const FXWindow* window,FXDragType type,FXuchar*& data,FXuint& size);
   void selectionGetTypes(const FXWindow* window,FXDragType*& types,FXuint& numtypes);
   void clipboardSetData(const FXWindow* window,FXDragType type,FXuchar* data,FXuint size);
   void clipboardGetData(const FXWindow* window,FXDragType type,FXuchar*& data,FXuint& size);
   void clipboardGetTypes(const FXWindow* window,FXDragType*& types,FXuint& numtypes);
+  void dragdropSetData(const FXWindow* window,FXDragType type,FXuchar* data,FXuint size);
+  void dragdropGetData(const FXWindow* window,FXDragType type,FXuchar*& data,FXuint& size);
+  void dragdropGetTypes(const FXWindow* window,FXDragType*& types,FXuint& numtypes);
 #ifndef WIN32
   void addRepaint(FXID win,FXint x,FXint y,FXint w,FXint h,FXbool synth=0);
   void removeRepaints(FXID win,FXint x,FXint y,FXint w,FXint h);
   void scrollRepaints(FXID win,FXint dx,FXint dy);
 #else
-  void leaveWindow(FXWindow *win,FXWindow *anc);
-  void enterWindow(FXWindow *win,FXWindow *anc);
   static long CALLBACK wndproc(FXID hwnd,unsigned int iMsg,unsigned int wParam,long lParam);
 protected:
   virtual long dispatchEvent(FXID hwnd,unsigned int iMsg,unsigned int wParam,long lParam);
@@ -349,13 +358,15 @@ protected:
 public:
   long onCmdQuit(FXObject*,FXSelector,void*);
   long onCmdDump(FXObject*,FXSelector,void*);
+  long onCmdHover(FXObject*,FXSelector,void*);
 
 public:
 
   /// Messages applications understand
   enum {
-    ID_QUIT=0,    /// Terminate the application normally
+    ID_QUIT=1,    /// Terminate the application normally
     ID_DUMP,      /// Dump the current widget tree
+    ID_HOVER,
     ID_LAST
     };
 
@@ -368,15 +379,16 @@ public:
 
   /**
   * Construct application object; the name and vendor strings are used
-  * as keys into the registry database for this application's settings
+  * as keys into the registry database for this application's settings.
+  * Only one single application object can be constructed.
   */
   FXApp(const FXString& name="Application",const FXString& vendor="FoxDefault");
 
   /// Get application name
-  FXString getAppName() const { return registry.getAppKey(); }
+  const FXString& getAppName() const { return registry.getAppKey(); }
 
   /// Get vendor name
-  FXString getVendorName() const { return registry.getVendorKey(); }
+  const FXString& getVendorName() const { return registry.getVendorKey(); }
 
   /// Connection to display; this is called by init()
   FXbool openDisplay(const FXchar* dpyname=NULL);
@@ -408,6 +420,9 @@ public:
   /// Get root Window
   FXRootWindow* getRootWindow() const { return root; }
 
+  /// Set root Window
+  void setRootWindow(FXRootWindow* rt);
+
   /// Get the window under the cursor, if any
   FXWindow *getCursorWindow() const { return cursorWindow; }
 
@@ -423,6 +438,15 @@ public:
   /// Find window from root x,y, starting from given window
   FXWindow* findWindowAt(FXint rx,FXint ry,FXID window=0) const;
 
+  /// Create application's windows
+  virtual void create();
+
+  /// Destroy application's windows
+  virtual void destroy();
+
+  /// Detach application's windows
+  virtual void detach();
+
   /**
   * Add timeout message to be sent to target object in ms milliseconds;
   * the timer fires only once after the interval expires.  The void* ptr
@@ -430,17 +454,12 @@ public:
   * handler.  If a timer with the same target and message already exists,
   * it will be rescheduled.
   */
-  FXTimer* addTimeout(FXObject* tgt,FXSelector sel,FXuint ms=1000,void* ptr=NULL);
+  void addTimeout(FXObject* tgt,FXSelector sel,FXuint ms=1000,void* ptr=NULL);
 
   /**
-  * Remove timeout identified by tgt and sel; returns NULL.
+  * Remove timeout identified by tgt and sel.
   */
-  FXTimer* removeTimeout(FXObject* tgt,FXSelector sel);
-
-  /**
-  * Remove timeout t; returns NULL.
-  */
-  FXTimer* removeTimeout(FXTimer *t);
+  void removeTimeout(FXObject* tgt,FXSelector sel);
 
   /**
   * Return TRUE if given timeout has been set
@@ -452,14 +471,12 @@ public:
   * If the timer is past due, 0 is returned.  If there is no such
   * timer, infinity (UINT_MAX) is returned.
   */
-  FXuint remainingTimeout(FXObject *tgt,FXSelector sel) const;
+  FXuint remainingTimeout(FXObject *tgt,FXSelector sel);
 
   /**
-  * Return, in ms, the time remaining until the given timer fires.
-  * If the timer is past due, 0 is returned.  If there is no such
-  * timer, infinity (UINT_MAX) is returned.
+  * Process any timeouts due at this time.
   */
-  FXuint remainingTimeout(FXTimer *t) const;
+  void handleTimeouts();
 
   /**
   * Add a idle processing message to be sent to target object when
@@ -468,17 +485,12 @@ public:
   * of the message handler. If a chore with the same target and message
   * already exists, it will be rescheduled.
   */
-  FXChore* addChore(FXObject* tgt,FXSelector sel,void *ptr=NULL);
+  void addChore(FXObject* tgt,FXSelector sel,void *ptr=NULL);
 
   /**
-  * Remove idle processing message identified by tgt and sel; returns NULL.
+  * Remove idle processing message identified by tgt and sel.
   */
-  FXChore* removeChore(FXObject* tgt,FXSelector sel);
-
-  /**
-  * Remove idle processing message; returns NULL.
-  */
-  FXChore* removeChore(FXChore *c);
+  void removeChore(FXObject* tgt,FXSelector sel);
 
   /**
   * Return TRUE if given chore has been set
@@ -511,20 +523,14 @@ public:
   */
   FXbool removeInput(FXInputHandle fd,FXuint mode);
 
-  /// Create application's windows
-  virtual void create();
-
-  /// Destroy application's windows
-  virtual void destroy();
-
-  /// Detach application's windows
-  virtual void detach();
+  /// Return key state
+  FXbool getKeyState(FXuint keysym) const;
 
   /// Peek to determine if there's an event
   FXbool peekEvent();
 
-  /// Perform one event dispatch
-  void runOneEvent();
+  /// Perform one event dispatch; return true if event was dispatched
+  FXbool runOneEvent(FXbool blocking=TRUE);
 
   /**
   * Run the main application event loop until stop() is called,
@@ -634,7 +640,12 @@ public:
   */
   virtual void exit(FXint code=0);
 
-  /// Get registry
+  /**
+  * Return a reference to the registry.  The registry keeps
+  * settings and configuration information for an application,
+  * which are automatically loaded when the application starts
+  * up, and saved when the application terminates.
+  */
   FXRegistry& reg(){ return registry; }
 
   /// Register new DND type
@@ -676,6 +687,20 @@ public:
   /// Change default cursor
   void setDefaultCursor(FXDefaultCursor which,FXCursor* cur);
 
+  /**
+  * Return a reference to the application-wide mutex.
+  * Normally, the main user interface thread holds this mutex,
+  * insuring that no other threads are modifying data during the
+  * processing of user interface messages. However, whenever the
+  * main user interface thread blocks for messages, it releases
+  * this mutex, to allow other threads to modify the same data.
+  * When a new message becomes available, the main user interface
+  * thread regains the mutex prior to dispatching the message.
+  * Other threads should hold this mutex only for short durations,
+  * so as to not starve the main user interface thread.
+  */
+  FXMutex& mutex(){ return appMutex; }
+
   /// Obtain application-wide settings
   FXuint getTypingSpeed() const { return typingSpeed; }
   FXuint getClickSpeed() const { return clickSpeed; }
@@ -713,6 +738,8 @@ public:
   FXColor getSelbackColor() const { return selbackColor; }
   FXColor getTipforeColor() const { return tipforeColor; }
   FXColor getTipbackColor() const { return tipbackColor; }
+  FXColor getSelMenuTextColor() const { return selMenuTextColor; }
+  FXColor getSelMenuBackColor() const { return selMenuBackColor; }
 
   /// Change default colors
   void setBorderColor(FXColor color);
@@ -725,6 +752,8 @@ public:
   void setSelbackColor(FXColor color);
   void setTipforeColor(FXColor color);
   void setTipbackColor(FXColor color);
+  void setSelMenuTextColor(FXColor color);
+  void setSelMenuBackColor(FXColor color);
 
   /// Save
   virtual void save(FXStream& store) const;
