@@ -3,7 +3,7 @@
 *                           S t r i n g   O b j e c t                           *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2005 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXString.h,v 1.65 2003/03/25 13:09:23 fox Exp $                          *
+* $Id: FXString.h,v 1.74 2005/01/16 16:06:06 fox Exp $                          *
 ********************************************************************************/
 #ifndef FXSTRING_H
 #define FXSTRING_H
@@ -61,7 +61,7 @@ public:
   void length(FXint len);
 
   /// Length of text
-  FXint length() const { return ((FXint*)str)[-1]; }
+  FXint length() const { return *(((FXint*)str)-1); }
 
   /// Get text contents
   const FXchar* text() const { return (const FXchar*)str; }
@@ -93,16 +93,16 @@ public:
   /// Convert to upper case
   FXString& upper();
 
-  /// Return num partition(s) of string separated by delimiter delim
+  /// Return num partition(s) beginning at start from a string separated by delimiters delim.
   FXString section(FXchar delim,FXint start,FXint num=1) const;
 
-  /// Return num partition(s) of string separated by delimiters in delim
-  FXString section(const FXchar* delim,FXint n,FXint start,FXint num=1) const;
+  /// Return num partition(s) beginning at start from a string separated by set of delimiters from delim of size n
+  FXString section(const FXchar* delim,FXint n,FXint start,FXint num) const;
 
-  /// Return num partition(s) of string separated by delimiters in delim
+  /// Return num partition(s) beginning at start from a string separated by set of delimiters from delim.
   FXString section(const FXchar* delim,FXint start,FXint num=1) const;
 
-  /// Return num partition(s) of string separated by delimiters in delim
+  /// Return num partition(s) beginning at start from a string separated by set of delimiters from delim.
   FXString section(const FXString& delim,FXint start,FXint num=1) const;
 
   /// Assign character c to this string
@@ -272,10 +272,10 @@ public:
   /// Find a character, searching backward; return position or -1
   FXint rfind(FXchar c,FXint pos=2147483647) const;
 
-  // Find n-th occurrence of character, searching forward; return position or -1
+  /// Find n-th occurrence of character, searching forward; return position or -1
   FXint find(FXchar c,FXint pos,FXint n) const;
 
-  // Find n-th occurrence of character, searching backward; return position or -1
+  /// Find n-th occurrence of character, searching backward; return position or -1
   FXint rfind(FXchar c,FXint pos,FXint n) const;
 
   /// Find a substring of length n, searching forward; return position or -1
@@ -438,6 +438,13 @@ public:
   friend FXAPI FXString FXStringVal(FXint num,FXint base=10);
   friend FXAPI FXString FXStringVal(FXuint num,FXint base=10);
 
+  /**
+  * Convert long integer number to a string, using the given number
+  * base, which must be between 2 and 16.
+  */
+  friend FXAPI FXString FXStringVal(FXlong num,FXint base=10);
+  friend FXAPI FXString FXStringVal(FXulong num,FXint base=10);
+
   /**
   * Convert real number to a string, using the given procision and
   * exponential notation mode, which may be FALSE (never), TRUE (always), or
@@ -450,6 +457,10 @@ public:
   friend FXAPI FXint FXIntVal(const FXString& s,FXint base=10);
   friend FXAPI FXuint FXUIntVal(const FXString& s,FXint base=10);
 
+  /// Convert string to long integer number, assuming given number base
+  friend FXAPI FXlong FXLongVal(const FXString& s,FXint base=10);
+  friend FXAPI FXulong FXULongVal(const FXString& s,FXint base=10);
+
   /// Convert string into real number
   friend FXAPI FXfloat FXFloatVal(const FXString& s);
   friend FXAPI FXdouble FXDoubleVal(const FXString& s);
