@@ -3,7 +3,7 @@
 *                         T r e e   L i s t   W i d g e t                       *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2005 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXTreeList.h,v 1.73 2003/10/10 18:06:38 fox Exp $                        *
+* $Id: FXTreeList.h,v 1.94.2.1 2005/04/10 03:24:47 fox Exp $                        *
 ********************************************************************************/
 #ifndef FXTREELIST_H
 #define FXTREELIST_H
@@ -57,32 +57,32 @@ class FXAPI FXTreeItem : public FXObject {
   friend class FXTreeList;
   friend class FXDirList;
 protected:
-  FXTreeItem *parent;
-  FXTreeItem *prev;
-  FXTreeItem *next;
-  FXTreeItem *first;
-  FXTreeItem *last;
-  FXString    label;
-  FXIcon     *openIcon;
-  FXIcon     *closedIcon;
-  void       *data;
-  FXuint      state;
+  FXTreeItem *parent;           // Parent item
+  FXTreeItem *prev;             // Previous item
+  FXTreeItem *next;             // Next item
+  FXTreeItem *first;            // First child item
+  FXTreeItem *last;             // Last child item
+  FXString    label;            // Text of item
+  FXIcon     *openIcon;         // Icon of item
+  FXIcon     *closedIcon;       // Icon of item
+  void       *data;             // Item user data pointer
+  FXuint      state;            // Item state flags
   FXint       x,y;
 protected:
   FXTreeItem():parent(NULL),prev(NULL),next(NULL),first(NULL),last(NULL),openIcon(NULL),closedIcon(NULL),data(NULL),state(0),x(0),y(0){}
   virtual void draw(const FXTreeList* list,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const;
   virtual FXint hitItem(const FXTreeList* list,FXint x,FXint y) const;
-protected:
+public:
   enum{
-    SELECTED        = 1,
-    FOCUS           = 2,
-    DISABLED        = 4,
-    OPENED          = 8,
-    EXPANDED        = 16,
-    HASITEMS        = 32,
-    DRAGGABLE       = 64,
-    OPENICONOWNED   = 128,
-    CLOSEDICONOWNED = 256
+    SELECTED        = 1,        /// Selected
+    FOCUS           = 2,        /// Focus
+    DISABLED        = 4,        /// Disabled
+    OPENED          = 8,        /// Opened
+    EXPANDED        = 16,       /// Expanded
+    HASITEMS        = 32,       /// Has virtual subitems
+    DRAGGABLE       = 64,       /// Draggable
+    OPENICONOWNED   = 128,      /// Open icon owned by item
+    CLOSEDICONOWNED = 256       /// Close icon owned by item
     };
 public:
 
@@ -114,19 +114,19 @@ public:
   FXint getNumChildren() const;
 
   /// Change item label
-  virtual void setText(const FXString& txt){ label=txt; }
+  virtual void setText(const FXString& txt);
 
   /// Get item label
   const FXString& getText() const { return label; }
 
-  /// Change open icon
-  virtual void setOpenIcon(FXIcon* icn){ openIcon=icn; }
+  /// Change open icon, deleting the old icon if it was owned
+  virtual void setOpenIcon(FXIcon* icn,FXbool owned=FALSE);
 
   /// Get open icon
   FXIcon* getOpenIcon() const { return openIcon; }
 
-  /// Change closed icon
-  virtual void setClosedIcon(FXIcon* icn){ closedIcon=icn; }
+  /// Change closed icon, deleting the old icon if it was owned
+  virtual void setClosedIcon(FXIcon* icn,FXbool owned=FALSE);
 
   /// Get closed icon
   FXIcon* getClosedIcon() const { return closedIcon; }
@@ -173,11 +173,11 @@ public:
   /// Return true if this item is draggable
   FXbool isDraggable() const { return (state&DRAGGABLE)!=0; }
 
-  /// Make open and or icon owned by the item
-  void setIconOwned(FXuint owned=(OPENICONOWNED|CLOSEDICONOWNED));
+  /// Return TRUE if subitems, real or imagined
+  FXbool hasItems() const { return (state&HASITEMS)!=0; }
 
-  /// Return open icon and closed icon ownership status
-  FXuint isIconOwned() const { return (state&(OPENICONOWNED|CLOSEDICONOWNED)); }
+  /// Change has items flag
+  void setHasItems(FXbool flag);
 
   /// Return true if descendent of parent item
   FXbool isChildOf(const FXTreeItem* item) const;
@@ -217,7 +217,26 @@ typedef FXint (*FXTreeListSortFunc)(const FXTreeItem*,const FXTreeItem*);
 
 
 
-/// Tree list Widget
+/**
+* A Tree List Widget organizes items in a hierarchical, tree-like fashion.
+* Subtrees can be collapsed or expanded by double-clicking on an item
+* or by clicking on the optional plus button in front of the item.
+* Each item may have a text and optional open-icon as well as a closed-icon.
+* The items may be connected by optional lines to show the hierarchical
+* relationship.
+* When an item's selected state changes, the treelist emits a SEL_SELECTED
+* or SEL_DESELECTED message.  If an item is opened or closed, a message
+* of type SEL_OPENED or SEL_CLOSED is sent.  When the subtree under an
+* item is expanded, a SEL_EXPANDED or SEL_COLLAPSED message is issued.
+* A change of the current item is signified by the SEL_CHANGED message.
+* In addition, the tree list sends SEL_COMMAND messages when the user
+* clicks on an item, and SEL_CLICKED, SEL_DOUBLECLICKED, and SEL_TRIPLECLICKED
+* when the user clicks once, twice, or thrice, respectively.
+* When items are added or removed, the tree list sends messages of the
+* type SEL_INSERTED or SEL_DELETED.
+* In each of these cases, a pointer to the item, if any, is passed in the
+* 3rd argument of the message.
+*/
 class FXAPI FXTreeList : public FXScrollArea {
   FXDECLARE(FXTreeList)
 protected:
@@ -240,6 +259,7 @@ protected:
   FXint              grabx;             // Grab point x
   FXint              graby;             // Grab point y
   FXString           lookup;            // Lookup string
+  FXString           tip;
   FXString           help;              // Help string
   FXbool             state;             // State of item
 protected:
@@ -285,8 +305,8 @@ public:
     };
 public:
 
-  /// Construct a tree list with nvis visible items; the tree list is initially empty
-  FXTreeList(FXComposite *p,FXint nvis,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=TREELIST_NORMAL,FXint x=0,FXint y=0,FXint w=0,FXint h=0);
+  /// Construct a new, initially empty tree list
+  FXTreeList(FXComposite *p,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=TREELIST_NORMAL,FXint x=0,FXint y=0,FXint w=0,FXint h=0);
 
   /// Create server-side resources
   virtual void create();
@@ -336,38 +356,32 @@ public:
   /// Return last root item
   FXTreeItem* getLastItem() const { return lastitem; }
 
-  /// Prepend new [possibly subclassed] item as first child of p
-  FXTreeItem* addItemFirst(FXTreeItem* p,FXTreeItem* item,FXbool notify=FALSE);
+  /// Fill tree list by appending items from array of strings
+  FXint fillItems(FXTreeItem* father,const FXchar** strings,FXIcon* oi=NULL,FXIcon* ci=NULL,void* ptr=NULL,FXbool notify=FALSE);
 
-  /// Prepend new item with given text and optional icon, and user-data pointer as first child of p
-  FXTreeItem* addItemFirst(FXTreeItem* p,const FXString& text,FXIcon* oi=NULL,FXIcon* ci=NULL,void* ptr=NULL,FXbool notify=FALSE);
+  /// Fill tree list by appending items from newline separated strings
+  FXint fillItems(FXTreeItem* father,const FXString& strings,FXIcon* oi=NULL,FXIcon* ci=NULL,void* ptr=NULL,FXbool notify=FALSE);
 
-  /// Append new [possibly subclassed] item as last child of p
-  FXTreeItem* addItemLast(FXTreeItem* p,FXTreeItem* item,FXbool notify=FALSE);
+  /// Insert [possibly subclassed] item under father before other item
+  FXTreeItem* insertItem(FXTreeItem* other,FXTreeItem* father,FXTreeItem* item,FXbool notify=FALSE);
 
-  /// Append new item with given text and optional icon, and user-data pointer as last child of p
-  FXTreeItem* addItemLast(FXTreeItem* p,const FXString& text,FXIcon* oi=NULL,FXIcon* ci=NULL,void* ptr=NULL,FXbool notify=FALSE);
+  /// Insert item with given text and optional icons, and user-data pointer under father before other item
+  FXTreeItem* insertItem(FXTreeItem* other,FXTreeItem* father,const FXString& text,FXIcon* oi=NULL,FXIcon* ci=NULL,void* ptr=NULL,FXbool notify=FALSE);
 
-  /// Append new [possibly subclassed] item after to other item
-  FXTreeItem* addItemAfter(FXTreeItem* other,FXTreeItem* item,FXbool notify=FALSE);
+  /// Append [possibly subclassed] item as last child of father
+  FXTreeItem* appendItem(FXTreeItem* father,FXTreeItem* item,FXbool notify=FALSE);
 
-  /// Append new item with given text and optional icon, and user-data pointer after to other item
-  FXTreeItem* addItemAfter(FXTreeItem* other,const FXString& text,FXIcon* oi=NULL,FXIcon* ci=NULL,void* ptr=NULL,FXbool notify=FALSE);
+  /// Append item with given text and optional icons, and user-data pointer as last child of father
+  FXTreeItem* appendItem(FXTreeItem* father,const FXString& text,FXIcon* oi=NULL,FXIcon* ci=NULL,void* ptr=NULL,FXbool notify=FALSE);
 
-  /// Prepend new [possibly subclassed] item prior to other item
-  FXTreeItem* addItemBefore(FXTreeItem* other,FXTreeItem* item,FXbool notify=FALSE);
+  /// Prepend [possibly subclassed] item as first child of father
+  FXTreeItem* prependItem(FXTreeItem* father,FXTreeItem* item,FXbool notify=FALSE);
 
-  /// Prepend new item with given text and optional icon, and user-data pointer prior to other item
-  FXTreeItem* addItemBefore(FXTreeItem* other,const FXString& text,FXIcon* oi=NULL,FXIcon* ci=NULL,void* ptr=NULL,FXbool notify=FALSE);
+  /// Prepend item with given text and optional icons, and user-data pointer as first child of father
+  FXTreeItem* prependItem(FXTreeItem* father,const FXString& text,FXIcon* oi=NULL,FXIcon* ci=NULL,void* ptr=NULL,FXbool notify=FALSE);
 
-  /// Reparent item under parent p
-  void reparentItem(FXTreeItem* item,FXTreeItem* p);
-
-  // Move item before other
-  FXTreeItem* moveItemBefore(FXTreeItem* other,FXTreeItem* item);
-
-  // Move item after other
-  FXTreeItem* moveItemAfter(FXTreeItem* other,FXTreeItem* item);
+  /// Move item under father before other item
+  FXTreeItem *moveItem(FXTreeItem* other,FXTreeItem* father,FXTreeItem* item);
 
   /// Remove item
   void removeItem(FXTreeItem* item,FXbool notify=FALSE);
@@ -385,16 +399,31 @@ public:
   FXint getItemHeight(const FXTreeItem* item) const { return item->getHeight(this); }
 
   /// Get item at x,y, if any
-  FXTreeItem* getItemAt(FXint x,FXint y) const;
+  virtual FXTreeItem* getItemAt(FXint x,FXint y) const;
 
   /**
-  * Search items for item by name, starting from start item; the
-  * flags argument controls the search direction, and case sensitivity.
+  * Search items by name, beginning from item start.  If the start item
+  * is NULL the search will start at the first, top-most item in the list.
+  * Flags may be SEARCH_FORWARD or SEARCH_BACKWARD to control the search
+  * direction; this can be combined with SEARCH_NOWRAP or SEARCH_WRAP
+  * to control whether the search wraps at the start or end of the list.
+  * The option SEARCH_IGNORECASE causes a case-insensitive match.  Finally,
+  * passing SEARCH_PREFIX causes searching for a prefix of the item name.
+  * Return NULL if no matching item is found.
   */
-  FXTreeItem* findItem(const FXString& text,FXTreeItem* start=NULL,FXuint flags=SEARCH_FORWARD|SEARCH_WRAP) const;
+  FXTreeItem* findItem(const FXString& name,FXTreeItem* start=NULL,FXuint flags=SEARCH_FORWARD|SEARCH_WRAP) const;
+
+  /**
+  * Search items by associated user data, beginning from item start. If the
+  * start item is NULL the search will start at the first, top-most item
+  * in the list.  Flags may be SEARCH_FORWARD or SEARCH_BACKWARD to control
+  * the search direction; this can be combined with SEARCH_NOWRAP or SEARCH_WRAP
+  * to control whether the search wraps at the start or end of the list.
+  */
+  FXTreeItem* findItemByData(const void *ptr,FXTreeItem* start=NULL,FXuint flags=SEARCH_FORWARD|SEARCH_WRAP) const;
 
   /// Scroll to make item visible
-  void makeItemVisible(FXTreeItem* item);
+  virtual void makeItemVisible(FXTreeItem* item);
 
   /// Change item's text
   void setItemText(FXTreeItem* item,const FXString& text);
@@ -403,13 +432,13 @@ public:
   FXString getItemText(const FXTreeItem* item) const;
 
   /// Change item's open icon
-  void setItemOpenIcon(FXTreeItem* item,FXIcon* icon);
+  void setItemOpenIcon(FXTreeItem* item,FXIcon* icon,FXbool owned=FALSE);
 
-  /// Return item's open icon
+  /// Return item's open icon, deleting the old icon if it was owned
   FXIcon* getItemOpenIcon(const FXTreeItem* item) const;
 
-  /// Chance item's closed icon
-  void setItemClosedIcon(FXTreeItem* item,FXIcon* icon);
+  /// Chance item's closed icon, deleting the old icon if it was owned
+  void setItemClosedIcon(FXTreeItem* item,FXIcon* icon,FXbool owned=FALSE);
 
   /// Return item's closed icon
   FXIcon* getItemClosedIcon(const FXTreeItem* item) const;
@@ -448,10 +477,10 @@ public:
   void updateItem(FXTreeItem* item) const;
 
   /// Enable item
-  FXbool enableItem(FXTreeItem* item);
+  virtual FXbool enableItem(FXTreeItem* item);
 
   /// Disable item
-  FXbool disableItem(FXTreeItem* item);
+  virtual FXbool disableItem(FXTreeItem* item);
 
   /// Select item
   virtual FXbool selectItem(FXTreeItem* item,FXbool notify=FALSE);
@@ -504,6 +533,12 @@ public:
   /// Sort children of item
   void sortChildItems(FXTreeItem* item);
 
+  /// Return sort function
+  FXTreeListSortFunc getSortFunc() const { return sortfunc; }
+
+  /// Change sort function
+  void setSortFunc(FXTreeListSortFunc func){ sortfunc=func; }
+
   /// Change text font
   void setFont(FXFont* fnt);
 
@@ -540,12 +575,6 @@ public:
   /// Change line color
   void setLineColor(FXColor clr);
 
-  /// Return sort function
-  FXTreeListSortFunc getSortFunc() const { return sortfunc; }
-
-  /// Change sort function
-  void setSortFunc(FXTreeListSortFunc func){ sortfunc=func; }
-
   /// Return list style
   FXuint getListStyle() const;
 
@@ -556,7 +585,7 @@ public:
   void setHelpText(const FXString& text);
 
   /// Get the status line help text for this list
-  FXString getHelpText() const { return help; }
+  const FXString& getHelpText() const { return help; }
 
   /// Save object to a stream
   virtual void save(FXStream& store) const;
